import java.util.Comparator;

public class Person implements Comparable<Person> {
    private static int count = 0;   // static (class) variables
    protected String name;    // instance variables
    protected int age;        // instance variables

    public Person() {
    	//System.out.println("Person Constructor"); // this("", 0); error: call to this must be first statemenht in constructor
    	this("", 0);
    }

    public Person(String name, int age) {
        count++;
        this.name = name;
        this.age = age;
    }
    
    public Person(Person other) {
        this(other.name, other.age);
    }

    public String getName() {
    	return name;
    }
    
    public void setName(String name) {
    	this.name = name;
    }

    public int getAge() {
    	return age;
    }

    public void setAge(int age) {
    	this.age = age;
    }

    @Override
    public boolean equals(Object other) {                            // Object.equals overriding
        if (other instanceof Person) {
          Person that = (Person) other;
          return that.canEqual(this) && this.getName().equals(that.getName()) && this.getAge() == that.getAge();
        }
        return false;
    }
    
    @Override 
    public int hashCode() {
        return (41 * getName().hashCode() + getAge());
    }

    public boolean canEqual(Object other) {
        return (other instanceof Person);
    }

    @Override
    public String toString() {                                       // Object.toString() overriding
        return "Person Name: " + name + " Age: " + age;
    }

    public void print() {                                            // instance methods
        System.out.println("Person Name: " + name + " Age: " + age);
    }
    
    public void set(String name, int age) {
        this.name = name;
        this.age = age;
    }
    
    public void set(Person other) {
        this.name = other.name;
        this.age = other.age;
    }
    
    public static void printCount() {		// static (class) methods
    	System.out.println("Person Count: " + count); 
    }

    public static int getCount() {			// static (class) methods
		return count; 
	}                  
    public static void setCount(int value) { // static (class) methods
		count = value; 
	}                  
    
	@Override
    public int compareTo(Person other) {
      String thisName = this.getName().toUpperCase();
      String otherName = ((Person)other).getName().toUpperCase();      
      return thisName.compareTo(otherName); //ascending order
      //return otherName.compareTo(thisName); //descending order
    }

    public static Comparator<Person> AgeComparator = new Comparator<Person>() {
		@Override
	    public int compare(Person p1, Person p2) {      
			int age1 = p1.getAge();
			int age2 = p2.getAge();
			return age1 - age2; //ascending order
			//return age2 - age1; //descending order
	    }
    };
}
